SystemOrganization addCategory: #'SCouchDB-Core'!

!Date methodsFor: '*SCouchDB-Core' stamp: 'rh 2/26/2010 23:30'!
asCouchDBString
	"Return a suitable string representation that sorts lexicographically:
		2008/06/09"

	^String streamContents: [:s |
		s nextPutAll: (self printFormat: #(3 2 1 $/ 1 1 2)).
		]
! !

!True class methodsFor: '*SCouchDB-Core' stamp: 'JesusMari 3/31/2013 11:29'!
instantiateFromCouchDocument: aJson
^true! !

!Number class methodsFor: '*SCouchDB-Core' stamp: 'Igor.Stasenko 1/12/2010 00:43'!
instantiateFromCouchDocument: json
	"Translate a json object, a couch document, into an instance of receiver.
	See #asCouchDocument. 
	
	Number instatiateFromCouchDocument: 123456 asCouchDocument
	"
	
	^ json at: 'sq_value' ! !

!Number methodsFor: '*SCouchDB-Core' stamp: 'Igor.Stasenko 1/12/2010 00:19'!
asCouchDocument

	^ JsonObject new
		at: 'sq_class' put: self class name;
		at: 'sq_value' put: self;
		yourself! !

!Dictionary class methodsFor: '*SCouchDB-Core' stamp: 'Igor.Stasenko 1/14/2010 00:42'!
instantiateFromCouchDocument: aJson

	"Translate a json object, a couch document, into an instance of receiver.
	See #asCouchDocument. "
	| associations dict |
	
	associations := aJson at: 'sq_values'.
	dict := self new.
	associations do: [:each | dict at: (each at: 'k') put: (each at: 'v') ].
	^ dict
	! !

!Dictionary methodsFor: '*SCouchDB-Core' stamp: 'Igor.Stasenko 1/14/2010 00:42'!
asCouchDocument
	| associations |
	
	associations := self associations collect: [:each |
		JsonObject new 
			at: 'k' put: each key;
			at: 'v' put: each value;
			yourself
		].
	^ JsonObject new
		at: 'sq_class' put: self class name;
		at: 'sq_values' put: associations;
		yourself! !

!UndefinedObject class methodsFor: '*SCouchDB-Core' stamp: 'Igor.Stasenko 1/12/2010 00:43'!
instantiateFromCouchDocument: json
	"Translate a json object, a couch document, into an instance of receiver.
	See #asCouchDocument. "
	
	"answer a sole instance of me"
	^ nil! !

!UndefinedObject methodsFor: '*SCouchDB-Core' stamp: 'Igor.Stasenko 1/12/2010 01:16'!
asCouchDocument

	^ JsonObject new
		at: 'sq_class' put: self class name;
		yourself! !

SharedPool subclass: #SCouchHTTPCodes
	instanceVariableNames: ''
	classVariableNames: 'HTTPCreated HTTPErrorConflict HTTPOK HTTPUnauthorized'
	poolDictionaries: ''
	category: 'SCouchDB-Core'!

!SCouchHTTPCodes class methodsFor: 'as yet unclassified' stamp: 'JesusMari 9/26/2013 09:33'!
initialize
	" self initialize "
	HTTPOK := 200.
	HTTPCreated := 201.
	HTTPUnauthorized:=401.
	HTTPErrorConflict := 409.! !

!False class methodsFor: '*SCouchDB-Core' stamp: 'JesusMari 3/31/2013 11:30'!
instantiateFromCouchDocument: aJson
^false! !

!DateAndTime class methodsFor: '*SCouchDB-Core' stamp: 'Igor.Stasenko 1/15/2010 14:05'!
fromCouchDBString: aString
	"Parse a string representation like:
	
		DateAndTime fromCouchDBString: '2008/06/09 13:52:11 -0130'
	
	This is not proper ISO8601 (which uses yyyy-mm-dd) but it can easily
	be used from javascript to create a Date instance."

	| date time stream offset instance sign |
	stream := aString readStream.
	stream skipSeparators.
	"note, #readFrom:pattern: may be missing"
	date := Date readFrom: stream pattern: 'yyyy/mm/dd'.
	stream skipSeparators.
	time := Time readFrom: stream.
	stream skipSeparators.
	sign := stream next.
	offset := (stream next: 2) asInteger hours + (stream next: 2) asInteger minutes.
	sign = $- ifTrue: [offset := offset negated].
	instance := self date: date time: time.
	^offset isZero ifTrue: [instance] ifFalse: [instance offset: offset]
! !

!DateAndTime methodsFor: '*SCouchDB-Core' stamp: 'Igor.Stasenko 1/15/2010 13:52'!
asCouchDBString
	"Return a suitable string representation that sorts lexicographically:
		2008/06/09 13:52:11 +0000
	
	This is not proper ISO8601 (which uses yyyy-mm-dd) but it can easily
	be used from javascript to create a Date instance."

	^String streamContents: [:s |
		s nextPutAll: (self date printFormat: #(3 2 1 $/ 1 1 2));
			space; nextPutAll: self time print24; space.
		offset negative ifFalse: [s nextPut: $+].
		s nextPutAll: (offset hours printPaddedWith: $0 to: 2);
			nextPutAll: (offset minutes printPaddedWith: $0 to: 2)]
! !

!Boolean methodsFor: '*SCouchDB-Core' stamp: 'Igor.Stasenko 1/12/2010 00:36'!
asCouchDocument

	^ JsonObject new
		at: 'sq_class' put: self class name;
		yourself! !

Stream subclass: #SCouchChunkedContentStream
	instanceVariableNames: 'socketstream chunkLen'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SCouchDB-Core'!

!SCouchChunkedContentStream class methodsFor: 'instance creation' stamp: 'Igor.Stasenko 1/13/2010 21:12'!
on: aSocketStream 
	^ self basicNew on: aSocketStream! !

!SCouchChunkedContentStream methodsFor: 'accessing' stamp: 'rh 3/20/2010 18:11'!
basicNext

	^self next! !

!SCouchChunkedContentStream methodsFor: 'accessing' stamp: 'rh 3/20/2010 18:11'!
isBinary

	^false! !

!SCouchChunkedContentStream methodsFor: 'accessing' stamp: 'Igor.Stasenko 1/13/2010 21:32'!
next
	chunkLen = 0 ifTrue: [
		socketstream skip: 2.  "skip cr/lf"
		self readChunkLength.
		chunkLen ifNil: [ ^ nil ].
	].

	chunkLen := chunkLen - 1.	
	^ socketstream next! !

!SCouchChunkedContentStream methodsFor: 'as yet unclassified' stamp: 'Igor.Stasenko 1/13/2010 21:22'!
on: aSocketStream
	socketstream := aSocketStream.
	
	self readChunkLength
! !

!SCouchChunkedContentStream methodsFor: 'accessing' stamp: 'Igor.Stasenko 1/13/2010 21:31'!
peek
	
	chunkLen = 0 ifTrue: [
		socketstream skip: 2.  "skip cr/lf"
		self readChunkLength.
	].

	^ socketstream peek! !

!SCouchChunkedContentStream methodsFor: 'accessing' stamp: 'Igor.Stasenko 4/7/2010 10:20'!
peekFor: anObject
	"Answer false and do not move over the next element if it is not equal to 
	the argument, anObject, or if the receiver is at the end. Answer true 
	and increment the position for accessing elements, if the next element is 
	equal to anObject."

	^ (self peek = anObject) and: [ self next. true ]! !

!SCouchChunkedContentStream methodsFor: 'as yet unclassified' stamp: 'Igor.Stasenko 1/13/2010 21:28'!
readChunkLength

	chunkLen := Number readFrom: socketstream nextLineCrLf asUppercase base: 16.
	
	chunkLen = 0 ifTrue: [
		"we finished"
		socketstream close.
		chunkLen := nil
		].
	! !

!Object class methodsFor: '*SCouchDB-Core' stamp: 'JesusMari 3/31/2013 11:27'!
instantiateFromCouchDocument: json
	"Translate a json object, a couch document, into an instance of receiver.
	See #asCouchDocument.
	
	Point instatiateFromCouchDocument: (1@2) asCouchDocument
	"

	| instance |
	self isVariable ifTrue: [ | values |
		values := json at: 'sq_values'.
		instance := self basicNew: values size.
		values withIndexDo: [:val :i | instance basicAt: i put: val ].
	] ifFalse: [ instance := self basicNew ].

	"fill ivars"
	
	self allInstVarNames do: [:iname |
		json at: iname ifPresent: [:val |
			instance instVarNamed: iname put: val
			  ]].
	^ instance! !

!Object methodsFor: '*SCouchDB-Core' stamp: 'Igor.Stasenko 1/13/2010 12:59'!
asCouchDocument
	" Answer a json object, representing a receiver as couchDB document"
	" A most generic implementation 
	
	 - sq_class holds a class name
	 - any instance variables is stored as name: value pairs
	 - sq_values holds an indexed variables array (if any)
	 - sq_value holds a single literal value (for integers, floats and booleans)
	
	Limitation: any values , held by vars should be able to jsonify (implement #jsonWriteOn: )

	Example: 
	   (1@2) asCouchDocument asJsonString
	   #(1 2 3) asCouchDocument asJsonString

	See also, #instantiateFromCouchDocument: at class side for reverse translation .
	"
	| json |
	json := JsonObject new.
	
	json at: 'sq_class' put: self class name.
	self class allInstVarNames do: [:name |
		json at: name put: (self instVarNamed: name) ].

	self class isVariable ifTrue: [ 
		json at: 'sq_values' put: 
			(( 1 to: self size ) collect: [:i | self at: i ])
	].

	^ json
! !

Object subclass: #SCouchDBAdaptor
	instanceVariableNames: 'host port resolvedHost httpResponseClass user password cookieAuthentication showProgress'
	classVariableNames: ''
	poolDictionaries: 'SCouchHTTPCodes'
	category: 'SCouchDB-Core'!

SCouchDBAdaptor class
	instanceVariableNames: 'authStrings'!

SCouchDBAdaptor class
	instanceVariableNames: 'authStrings'!

!SCouchDBAdaptor class methodsFor: 'as yet unclassified' stamp: 'JesusMari 3/22/2012 09:55'!
addAuthCookieToHost: aHost user:aUser cookie:aCookie
authStrings ifNil:[authStrings:= Dictionary new].
authStrings at:(aHost,aUser) put: aCookie.! !

!SCouchDBAdaptor class methodsFor: 'as yet unclassified' stamp: 'JesusMari 3/22/2012 13:51'!
getAuthCookieToHost: aHost user:aUser 
authStrings ifNil:[authStrings:= Dictionary new].
^ authStrings at:(aHost,aUser) ifAbsent:[^nil]! !

!SCouchDBAdaptor class methodsFor: 'as yet unclassified' stamp: 'JesusMari 3/22/2012 13:59'!
removeAuthCookieToHost: aHost user:aUser
authStrings ifNil:[authStrings:= Dictionary new].
authStrings removeKey:(aHost,aUser) ifAbsent:[^nil]! !

!SCouchDBAdaptor methodsFor: 'server-level' stamp: 'Igor.Stasenko 12/27/2009 09:57'!
activeTasks
	^ self getJson: '/_active_tasks'! !

!SCouchDBAdaptor methodsFor: 'myMethods' stamp: 'JesusMari 3/22/2012 09:37'!
authenticatedRequest
^ ((user notNil) and:[ password notNil])! !

!SCouchDBAdaptor methodsFor: 'as yet unclassified' stamp: 'JesusMari 9/26/2013 10:34'!
boundary
^'abc123'! !

!SCouchDBAdaptor methodsFor: 'myMethods' stamp: 'JesusMari 3/22/2012 13:51'!
buildAuthString
|authString|
authString:= self class getAuthCookieToHost:host user:user.

authString ifNil:[
	authString:=
		'name=',user,
		'&password=', password.
		^authString
		].
^nil

! !

!SCouchDBAdaptor methodsFor: 'server-config' stamp: 'Igor.Stasenko 12/27/2009 09:52'!
config
	^ self getJson: '/_config'! !

!SCouchDBAdaptor methodsFor: '!!private!!' stamp: ''!
connectUsingHeader: aHeader

	^ self connectUsingHeader: aHeader content: nil! !

!SCouchDBAdaptor methodsFor: '!!private!!' stamp: ''!
connectUsingHeader: aHeader content: aContent
	| stream line response lines i aBlock |
	stream := SocketStream openConnectionToHost: self resolvedHost port: self port.
	stream nextPutAll: aHeader.

	aContent ifNotNil: [ 
		"aContent is a Multipart/related content"
		(aContent isKindOf: OrderedCollection) 
		ifTrue:[
			stream nextPutAll: '--'; nextPutAll:self boundary; crlf.
			i:=1.
			aContent do:[:aPart|
				(i>1) ifTrue:[stream crlf;crlf].
				stream nextPutAll: aPart upToEnd.
				i:=i+1.
				stream crlf;nextPutAll: '--';nextPutAll: self boundary.
				aPart close.
				].
			stream nextPutAll: '--'.
		]
		ifFalse:[
			"aContent is a binary stream"
			(aContent isKindOf: FileStream) ifTrue:[
				stream nextPutAll:aContent upToEnd
			] ifFalse:[ 
				"aContent is a JSon Object in String form "
				showProgress 
				ifFalse:[stream  nextPutAll:aContent]
				ifTrue:[aContent do:[:aByte|stream  nextPut:aByte] displayingProgress:('Uploading to ' , self host, '...')]]
			]
		 ].
	
	stream flush.
	line := stream nextLineCrLf.
	(line beginsWith: 'HTTP/1.1') ifFalse: [
		^ self error: 'Malformed response header' ].

	response := self newHttpResponse.

	"  Integer readFrom: ('HTTP/1.1 200 OK' readStream upTo: $ ;yourself) "
	response code: ( 
		Integer readFrom: (line readStream upTo: Character space; yourself)).

	lines := OrderedCollection new.
	[ (line := stream nextLineCrLf) notEmpty ] whileTrue: [
		lines add: line ].
	response parseHeaderLines: lines.
	response contentStream: stream.
	^ response
! !

!SCouchDBAdaptor methodsFor: 'http api' stamp: 'JesusMari 4/9/2012 14:49'!
copy: path to:aDest

	^ self connectUsingHeader: (self formRequestHeader: 'COPY' path: path content:nil type:nil destination:aDest).! !

!SCouchDBAdaptor methodsFor: 'error handling' stamp: 'Igor.Stasenko 1/8/2010 19:38'!
couchDBError: jobject 
	^ SCouchDBError signal: 
		'Error: ', (jobject at: 'error') , ' reason: ', (jobject at: 'reason')! !

!SCouchDBAdaptor methodsFor: 'db api' stamp: 'Igor.Stasenko 4/8/2010 12:00'!
createDatabase: aName
	| result |
	aName do: [ :c | ('abcdefghijklmnopqrstuvwxyz0123456789_$()+-/`' includes: c)  
		ifFalse: [ ^ self error: 'Invalid database name' ]].

	result := self put: '/' , aName encodeForHTTP , '/'.
	
	result code = HTTPCreated ifTrue: [
		^ self newDatabase name: aName
		].
	
	^ self couchDBError: (self readJsonFrom: result)
	! !

!SCouchDBAdaptor methodsFor: 'server-level' stamp: 'Igor.Stasenko 12/27/2009 07:05'!
databases
	| response |
	response := self get: '/_all_dbs'.
	
	^ self readJsonFrom: response	! !

!SCouchDBAdaptor methodsFor: 'http api' stamp: 'Agnaldo de Oliveira 11/10/2010 19:00'!
delete: path 

	^ self connectUsingHeader: (self formRequestHeader: 'DELETE' path: path).! !

!SCouchDBAdaptor methodsFor: 'db api' stamp: 'JesusMari 3/22/2012 10:35'!
deleteDatabase: aName
	| header result |
	header := self formRequestHeader: 'DELETE' path: '/', aName encodeForHTTP , '/'.
	result := self connectUsingHeader: header.

	(result code = HTTPOK )
		ifFalse: [  ^ self couchDBError: (self readJsonFrom: result) ].

! !

!SCouchDBAdaptor methodsFor: 'http api' stamp: 'Agnaldo de Oliveira 11/10/2010 19:23'!
deleteDocument: path 
	"read a single resposnse for deletion of document"

	^ self readJsonFrom: (self delete: path)! !

!SCouchDBAdaptor methodsFor: 'db api' stamp: 'gk 5/20/2010 16:25'!
ensureDatabase: aName
	"Either create new or answer an existing database."
	
	(self hasDatabase: aName) ifFalse: [ ^self createDatabase: aName ].
	^ self newDatabase name: aName! !

!SCouchDBAdaptor methodsFor: '!!private!!' stamp: ''!
formRequestHeader: type path: aPath

	^ self formRequestHeader: type path: aPath content: nil ! !

!SCouchDBAdaptor methodsFor: '!!private!!' stamp: ''!
formRequestHeader: type path: aPath content: content

	^self formRequestHeader:type path:aPath content:content type:'application/json; charset=utf-8' destination:nil
! !

!SCouchDBAdaptor methodsFor: '!!private!!' stamp: ''!
formRequestHeader: type path: aPath content: content type:contentType 
^self formRequestHeader: type path: aPath content: content type:contentType destination:nil
! !

!SCouchDBAdaptor methodsFor: '!!private!!' stamp: ''!
formRequestHeader: type path: aPath content: content type:contentType destination:aDest

	| authCookie |
	
	^ String streamContents: [:str |
		str nextPutAll: type ; space; nextPutAll: aPath; space; nextPutAll: 'HTTP/1.1'; crlf;
			nextPutAll: 'Host: ', host , ':', port asString; crlf;
			nextPutAll: 'Accept: application/json, */*';crlf.
			(content isKindOf:OrderedCollection) ifFalse:[str nextPutAll: 'Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7'; crlf].
			aDest ifNotNil:[str nextPutAll:'Destination:', aDest;crlf].
			authCookie:= self class getAuthCookieToHost:host user:user.
			authCookie notNil ifTrue:[str nextPutAll: ('Cookie: ' , authCookie);crlf].
			content ifNotNil: [
				(content isKindOf:OrderedCollection) 
				ifFalse:[
				str nextPutAll: 'Content-length: ' , content size asString; crlf;
					nextPutAll: ('Content-Type:', contentType); crlf.
				]
				ifTrue:[
					|le|
					le:=0.
					content do:[:aPart| le:= le + aPart size.].
		
					le:= le + 
						 (self boundary size + 4 )+"first line"
						((content size ) *(self boundary size + 8 )) - 2."3 crlf + --" 
						
					str nextPutAll: 'Content-length: ' , le asString; crlf;
						nextPutAll: ('Content-Type:', contentType); crlf.]].
		
		str crlf.
	]

! !

!SCouchDBAdaptor methodsFor: 'http api' stamp: 'Igor.Stasenko 12/27/2009 05:27'!
get: path 

	^ self connectUsingHeader: (self formRequestHeader: 'GET' path: path).! !

!SCouchDBAdaptor methodsFor: 'http api' stamp: 'Igor.Stasenko 1/13/2010 22:44'!
getJson: path 
	"read a single json document"

	^ self readJsonFrom: (self get: path)! !

!SCouchDBAdaptor methodsFor: 'db api' stamp: 'gk 5/20/2010 16:24'!
hasDatabase: aName
	
	^self databases includes: aName! !

!SCouchDBAdaptor methodsFor: 'http api' stamp: 'JesusMari 4/9/2012 11:29'!
head: path 

	^ self connectUsingHeader: (self formRequestHeader: 'HEAD' path: path).! !

!SCouchDBAdaptor methodsFor: 'accessing' stamp: 'Igor.Stasenko 12/27/2009 06:29'!
host
	^ host! !

!SCouchDBAdaptor methodsFor: 'initialize-release' stamp: 'gk 1/13/2010 11:52'!
host: aHost
	^self host: aHost port: 5984! !

!SCouchDBAdaptor methodsFor: 'initialize-release' stamp: 'Igor.Stasenko 12/27/2009 07:42'!
host: aHost port: aPort
	host := aHost.
	port := aPort.
	resolvedHost := nil.! !

!SCouchDBAdaptor methodsFor: 'authentication' stamp: 'JesusMari 3/22/2012 11:01'!
host: aHost port: aPort userName:aUserName password:aPassword

	host:=aHost.
	port := aPort.
	user:=aUserName.
	password:=aPassword.
	resolvedHost := nil.
	
	self session! !

!SCouchDBAdaptor methodsFor: 'streaming mode' stamp: 'Igor.Stasenko 1/8/2010 20:00'!
httpResponseClass: aResponseClass
	httpResponseClass := aResponseClass! !

!SCouchDBAdaptor methodsFor: 'server-level' stamp: 'JesusMari 5/5/2012 12:45'!
info
	^ self getJson: '/'! !

!SCouchDBAdaptor methodsFor: 'as yet unclassified' stamp: 'JesusMari 8/30/2013 09:55'!
initialize 
showProgress:=false! !

!SCouchDBAdaptor methodsFor: 'authentication' stamp: 'JesusMari 5/5/2012 13:03'!
logout
	"read a single resposnse for deletion of document"
	| resp |
	resp:=self readJsonFrom: (self delete: '/_session').
	self class removeAuthCookieToHost:host user:user.
	^ resp! !

!SCouchDBAdaptor methodsFor: 'accessing' stamp: 'Igor.Stasenko 1/8/2010 19:39'!
newDatabase
	"Answer a new instance of couch database wrapper"
	^ SCouchDatabase new adaptor: self! !

!SCouchDBAdaptor methodsFor: 'streaming mode' stamp: 'Igor.Stasenko 1/8/2010 20:02'!
newHttpResponse

	^ (httpResponseClass ifNil: [ SCouchHTTPResponse ]) new! !

!SCouchDBAdaptor methodsFor: 'accessing' stamp: 'ZekeriyaKoc 4/8/2010 10:01'!
newUUID
	^ (self uuids at: 'uuids') at: 1! !

!SCouchDBAdaptor methodsFor: 'accessing' stamp: 'ZekeriyaKoc 4/8/2010 10:00'!
newUUIDs: aNumber
	^ (self uuids: aNumber) at: 'uuids'! !

!SCouchDBAdaptor methodsFor: 'myMethods' stamp: 'JesusMari 3/22/2012 13:40'!
parseCookie: aCookie
| collection |
collection:=aCookie subStrings:';'.
(collection size>0) ifTrue:[
^collection at:1]! !

!SCouchDBAdaptor methodsFor: 'authentication' stamp: 'JesusMari 3/21/2012 19:17'!
password
	^ password! !

!SCouchDBAdaptor methodsFor: 'authentication' stamp: 'JesusMari 3/21/2012 19:17'!
password: anObject
	password := anObject! !

!SCouchDBAdaptor methodsFor: 'accessing' stamp: 'Igor.Stasenko 12/27/2009 06:29'!
port
	^ port! !

!SCouchDBAdaptor methodsFor: 'http api' stamp: 'rh 2/20/2010 13:20'!
post: path content: aContent

	^ self connectUsingHeader: (self formRequestHeader: 'POST' path: path content: aContent) content: aContent.! !

!SCouchDBAdaptor methodsFor: 'http api' stamp: 'JesusMari 3/22/2012 11:43'!
post: path content: aContent type:aContentType

	^ self connectUsingHeader: (self formRequestHeader: 'POST' path: path content: aContent type:aContentType) content: aContent.! !

!SCouchDBAdaptor methodsFor: 'http api' stamp: 'Igor.Stasenko 12/27/2009 07:23'!
put: path 

	^ self put: path content: nil! !

!SCouchDBAdaptor methodsFor: 'http api' stamp: 'Igor.Stasenko 12/27/2009 07:23'!
put: path content: aContent

	^ self connectUsingHeader: (self formRequestHeader: 'PUT' path: path content: aContent) content: aContent.! !

!SCouchDBAdaptor methodsFor: 'http api' stamp: 'JesusMariAguirre 5/14/2012 14:35'!
put: path content: aContent type:mime

	^ self connectUsingHeader: 
						(self formRequestHeader: 'PUT' 
								path: path 
								content: aContent
								type:mime
								) 
			content: aContent .! !

!SCouchDBAdaptor methodsFor: '!!private!!' stamp: ''!
readETagFrom: response	
	| result |
	 ('*application/json*' match: response contentType) ifFalse: [
		self error: 'response is not json'.
		].
	
	result := Json readFrom: (SCouchUTF8Stream on: response readStream).
	" close steam after use "
	response closeStream.
	^ result! !

!SCouchDBAdaptor methodsFor: '!!private!!' stamp: ''!
readJsonFrom: response	
	| result |
	 ('*application/json*' match: response contentType) ifFalse: [
		self error: 'response is not json'.
		].
	
	result := Json readFrom: (SCouchUTF8Stream on: response readStream).
	" close steam after use "
	response closeStream.
	^ result! !

!SCouchDBAdaptor methodsFor: '!!private!!' stamp: ''!
resolvedHost 

	^ resolvedHost ifNil: [
		resolvedHost := (NetNameResolver addressForName: self host) ]! !

!SCouchDBAdaptor methodsFor: 'authentication' stamp: 'JesusMari 3/22/2012 14:00'!
session
	| response authCookie|
	self class removeAuthCookieToHost:host user:user.
	response := self post: '/_session' content:self buildAuthString type:'application/x-www-form-urlencoded'.
	authCookie:=response header at:'set-cookie' ifAbsent:[nil].
	self class addAuthCookieToHost:host user:user cookie:(self parseCookie:authCookie).

	^ self readJsonFrom: response	! !

!SCouchDBAdaptor methodsFor: 'authentication' stamp: 'JesusMari 5/5/2012 12:54'!
sessionInfo
	| response authCookie|

	response := self get: '/_session'.
	authCookie:=response header at:'set-cookie' ifAbsent:[nil].
	
	^ self readJsonFrom: response	
	
! !

!SCouchDBAdaptor methodsFor: 'accessing' stamp: 'JesusMari 8/30/2013 09:55'!
showProgress:aBoolean
	showProgress:=aBoolean! !

!SCouchDBAdaptor methodsFor: 'server-level' stamp: 'Igor.Stasenko 12/27/2009 09:55'!
statistics
	^ self getJson: '/_stats'! !

!SCouchDBAdaptor methodsFor: 'authentication' stamp: 'JesusMari 3/21/2012 19:16'!
user
	^ user! !

!SCouchDBAdaptor methodsFor: 'authentication' stamp: 'JesusMari 3/21/2012 19:16'!
user: anObject
	user := anObject! !

!SCouchDBAdaptor methodsFor: 'authentication' stamp: 'JesusMari 5/5/2012 13:12'!
users

	^self readJsonFrom: (self get: '/_users').
! !

!SCouchDBAdaptor methodsFor: 'server-level' stamp: 'ZekeriyaKoc 4/7/2010 17:06'!
uuids
	^ self getJson: '/_uuids'! !

!SCouchDBAdaptor methodsFor: 'server-level' stamp: 'ZekeriyaKoc 4/8/2010 09:51'!
uuids: aNumber
	^ self getJson: '/_uuids?count=', aNumber printString! !

Object subclass: #SCouchDatabase
	instanceVariableNames: 'adaptor name'
	classVariableNames: ''
	poolDictionaries: 'SCouchHTTPCodes'
	category: 'SCouchDB-Core'!

!SCouchDatabase methodsFor: 'accessing' stamp: 'Igor.Stasenko 12/27/2009 07:40'!
adaptor
	"Answer the value of adaptor"

	^ adaptor! !

!SCouchDatabase methodsFor: 'accessing' stamp: 'Igor.Stasenko 12/27/2009 07:40'!
adaptor: anObject
	"Set the value of adaptor"

	adaptor := anObject! !

!SCouchDatabase methodsFor: 'accessing' stamp: 'JesusMari 4/6/2012 01:59'!
all

	| json |
	json := self allDocuments.
! !

!SCouchDatabase methodsFor: 'accessing' stamp: 'gk 5/20/2010 16:22'!
allDocuments
	| result |
	result := self getJson: '_all_docs?include_docs=true'.
	^ (result at: 'rows') collect: [:row | row at: 'doc']! !

!SCouchDatabase methodsFor: 'accessing' stamp: 'gk 5/20/2010 16:21'!
allDocumentsAs: aClass
	| result |
	result := self getJson: '_all_docs?include_docs=true'.
	^ (result at: 'rows') collect: [:row | aClass fromCouchDocument: (row at: 'doc')]! !

!SCouchDatabase methodsFor: 'accessing' stamp: 'Igor.Stasenko 1/11/2010 23:57'!
at: id 

	^ self at: id ifAbsent: [ self error: 'Document with given id not found' ]! !

!SCouchDatabase methodsFor: 'accessing' stamp: 'gk 1/13/2010 12:09'!
at: id ifAbsent: aBlock
	
	^self documentToObject: (self documentAt: id ifAbsent: [ ^ aBlock value ])! !

!SCouchDatabase methodsFor: 'accessing' stamp: 'Igor.Stasenko 1/9/2010 17:25'!
at: key put: object

	^ self documentAt: key put: object asCouchDocument! !

!SCouchDatabase methodsFor: 'as yet unclassified' stamp: 'JesusMari 5/27/2012 18:57'!
deleteDocument: path 

	^ self deleteDocument: path  revision: (self documentCurrentRevAt:path) asString! !

!SCouchDatabase methodsFor: 'as yet unclassified' stamp: 'JesusMari 5/27/2012 18:51'!
deleteDocument: path revision: aRev

	^ adaptor deleteDocument:  self namePrefix , path ,'?rev=', aRev! !

!SCouchDatabase methodsFor: 'accessing' stamp: 'JesusMari 3/22/2012 14:15'!
documentAt: id

	"read a single json document"
	| response result |
	
	(id asString beginsWith: '_design') 
	ifTrue: 
	[response := adaptor get: self namePrefix , id asString]	
	ifFalse:
	[response := adaptor get: self namePrefix , id asString encodeForHTTP].

	result := adaptor readJsonFrom: response.
	response code = HTTPOK ifTrue: [ ^ result ].
	
	^ adaptor couchDBError: result! !

!SCouchDatabase methodsFor: 'accessing' stamp: 'JesusMari 9/25/2013 11:18'!
documentAt: id attach: content
	self documentAt: id attach: content rev:(self documentCurrentRevAt: id).! !

!SCouchDatabase methodsFor: 'accessing' stamp: 'JesusMari 9/26/2013 09:45'!
documentAt: id attach: content rev:oldDocRev
	| response code |
	
	(id asString beginsWith: '_') 
		ifTrue: 
			[response := self adaptor
							put: self namePrefix, id asString  , '/', (content name ,'?rev=', oldDocRev encodeForHTTP)
							content: content  stream 
							type:content contentType ]
		ifFalse:
			[response := self adaptor
							put: (self namePrefix,  id asString , '/', content name ,'?rev=', oldDocRev) encodeForHTTP 
							content: content  stream
							type:content contentType ].
	
	code:= response code.
	
	code = HTTPCreated ifTrue: [ ^adaptor readJsonFrom:  response ].
	code = HTTPOK ifTrue: [ ^ adaptor readJsonFrom: response ].
	code = HTTPUnauthorized  ifTrue:[self error:'Unauthorized, login to CouchDB'].
	self error: 'Error Attaching file'
! !

!SCouchDatabase methodsFor: 'accessing' stamp: 'JesusMariAguirre 5/14/2012 09:56'!
documentAt: id ifAbsent: aBlock

	"read a single json document"
	| response |
	
	(id asString beginsWith: '_') 
	ifTrue: 
	[response := adaptor get: self namePrefix , id asString]	
	ifFalse:
	[response := adaptor get: self namePrefix , id asString encodeForHTTP.].
	response code = HTTPOK ifTrue: [
		^ adaptor readJsonFrom: response.
		].
	
	^ aBlock value! !

!SCouchDatabase methodsFor: 'accessing' stamp: 'JesusMariAguirre 5/14/2012 09:56'!
documentAt: id ifAbsentPut: aBlockWithJsonObject

	"read a single json document"
	| response |
	(id asString beginsWith: '_') 
	ifTrue: 
	[response := adaptor get: self namePrefix , id asString]	
	ifFalse:
	[response := adaptor get:  self namePrefix , id asString encodeForHTTP].

	response code = HTTPOK ifTrue: [
		^ adaptor readJsonFrom: response 
		].
	
	^self documentAt: id put: aBlockWithJsonObject value.
! !

!SCouchDatabase methodsFor: 'accessing' stamp: 'JesusMari 9/26/2013 10:44'!
documentAt: id put: jsonObject
	| response code content |

	((jsonObject includeDocs isNil) or:[jsonObject includeDocs])
	ifTrue:[
		content := self utf8Encode: jsonObject asJsonString.
		(id asString beginsWith: '_') 
		ifTrue: 
			[response := self put: id asString  content: content]	
		ifFalse:
			[response := self put: id asString encodeForHTTP content: content]
		]
	ifFalse:[
		(id asString beginsWith: '_') 
		ifTrue: 
			[response := self adaptor
							put: self namePrefix, id asString  
							content: (self arrayOfMultipartsFrom:jsonObject)
							type:' multipart/related;boundary="', self boundary ,'"']
		ifFalse:
			[response := self adaptor
							put: self namePrefix,  id asString encodeForHTTP 
							content: (self arrayOfMultipartsFrom:jsonObject)
							type:' multipart/related;boundary="', self boundary ,'"']
		].
	code:= response code.
	
	code = HTTPCreated ifTrue: [ ^adaptor readJsonFrom:  response ].
	code = HTTPOK ifTrue: [ ^ adaptor readJsonFrom: response ].
	code = HTTPUnauthorized  ifTrue:[self error:'Unauthorized, login to CouchDB'].
	code = HTTPErrorConflict ifTrue: [ 
		"document already exists"
		| oldDocRev | 
		response closeStream.
		"documentCurrentRevAt: uses HEAD method only retries the rev field"
		oldDocRev := self documentCurrentRevAt: id.
		jsonObject at: '_rev' put: oldDocRev. 	
		" retry "
		^ self documentAt: id put: jsonObject
	].
	
	self error: 'Boom'
! !

!SCouchDatabase methodsFor: 'accessing' stamp: 'Igor.Stasenko 4/8/2010 11:56'!
documentAt: id rev: rev

	| response result |
	response := adaptor get: self namePrefix , id asString encodeForHTTP ,
		'?rev=' , rev asString encodeForHTTP.

	result := adaptor readJsonFrom: response.
	response code = HTTPOK ifTrue: [ ^ result ].
	^ adaptor couchDBError: result! !

!SCouchDatabase methodsFor: 'accessing' stamp: 'JesusMari 9/25/2013 11:02'!
documentAt: id update: newDoc
	| oldDocRev|
	oldDocRev:=self documentCurrentRevAt:id.
	newDoc rev:oldDocRev.
	self documentAt:id put:newDoc.
	
	
	

	
! !

!SCouchDatabase methodsFor: 'accessing' stamp: 'gk 5/20/2010 16:01'!
documentPut: jsonObject
	| response code content |
	
	content := self utf8Encode: jsonObject asJsonString.
	response := self post: content.
	
	code := response code.
	
	code = HTTPCreated ifTrue: [ ^adaptor readJsonFrom:  response ].
	code = HTTPOK ifTrue: [ ^ adaptor readJsonFrom: response ].

	self error: 'Boom'

! !

!SCouchDatabase methodsFor: 'accessing' stamp: 'gk 1/13/2010 12:08'!
documentToObject: json

	| className binding |

	className := json at: 'sq_class'.

	"we're intentionally avoid using any globals here, such as Smalltalk"
	binding := self class environment bindingOf: className asSymbol.
	
	binding ifNil: [ ^ self error: 'unable to instantiate - class not found' ].
	
	^ binding value instantiateFromCouchDocument: json.
! !

!SCouchDatabase methodsFor: 'as yet unclassified' stamp: 'Igor.Stasenko 1/9/2010 17:11'!
getJson: path 

	^ adaptor getJson:  self namePrefix , path! !

!SCouchDatabase methodsFor: 'accessing' stamp: 'Igor.Stasenko 1/9/2010 17:11'!
info
	"retrieve my info from server"
	
	^ adaptor getJson: self namePrefix! !

!SCouchDatabase methodsFor: 'accessing' stamp: 'Igor.Stasenko 12/27/2009 07:40'!
name
	^ name! !

!SCouchDatabase methodsFor: 'accessing' stamp: 'Igor.Stasenko 12/27/2009 07:40'!
name: aString
	name := aString! !

!SCouchDatabase methodsFor: 'accessing' stamp: 'Igor.Stasenko 1/9/2010 17:10'!
namePrefix
	"path prefix for db name , 
	could be optimized later to use lazy-initialized ivar"
	
	^ '/' , self name , '/'
! !

!SCouchDatabase methodsFor: 'as yet unclassified' stamp: 'rh 2/20/2010 13:35'!
post: aContent

	^ adaptor post: self namePrefix content: aContent! !

!SCouchDatabase methodsFor: 'accessing' stamp: 'gk 5/20/2010 16:01'!
put: object

	^ self documentPut: object asCouchDocument! !

!SCouchDatabase methodsFor: 'as yet unclassified' stamp: 'Igor.Stasenko 1/9/2010 17:11'!
put: aReq content: aContent

	^ adaptor put: self namePrefix, aReq content: aContent! !

!SCouchDatabase methodsFor: 'accessing' stamp: 'Igor.Stasenko 5/13/2010 05:01'!
utf8Encode: aString 

	^ ByteString streamContents: [:str | aString do: [:ch | self utf8Encode: ch to: str ]]

! !

!SCouchDatabase methodsFor: 'accessing' stamp: 'Igor.Stasenko 5/13/2010 05:00'!
utf8Encode: aChar to: aStream
	| u |
	u := aChar asUnicode.
	u < 128 ifTrue: [ ^ aStream nextPut: aChar].
	u < 16r800 ifTrue: [
		^ aStream 
			nextPut: (Character value: (2r11000000 + (u >> 6)));
			nextPut: (Character value: (2r10000000 + (u bitAnd: 2r111111))) ].
	u < 16r10000 ifTrue: [
		^ aStream 
			nextPut: (Character value: (2r11100000 + (u >> 12)));
			nextPut: (Character value: (2r10000000 + ((u >> 6)bitAnd: 2r111111)));
			nextPut: (Character value: (2r10000000 + (u bitAnd: 2r111111))) ].
	u <= 16r10FFFF ifTrue: [
		^ aStream 
			nextPut: (Character value: (2r11110000 + (u >> 18)));
			nextPut: (Character value: (2r10000000 + ((u >> 12)bitAnd: 2r111111)));
			nextPut: (Character value: (2r10000000 + ((u >> 6)bitAnd: 2r111111)));
			nextPut: (Character value: (2r10000000 + (u bitAnd: 2r111111))) ].

	self error: 'Invalid utf-8 value'
! !

Object subclass: #SCouchHTTPResponse
	instanceVariableNames: 'header code content cstream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SCouchDB-Core'!

!SCouchHTTPResponse methodsFor: 'accessing' stamp: 'Igor.Stasenko 1/13/2010 20:42'!
closeStream
	"close an external connection, useful only for streaming response"
	
	cstream close! !

!SCouchHTTPResponse methodsFor: 'accessing' stamp: 'Igor.Stasenko 12/27/2009 06:20'!
code
	"Answer the value of code"

	^ code! !

!SCouchHTTPResponse methodsFor: 'accessing' stamp: 'Igor.Stasenko 12/27/2009 06:20'!
code: anObject
	"Set the value of code"

	code := anObject! !

!SCouchHTTPResponse methodsFor: 'accessing' stamp: 'Igor.Stasenko 1/13/2010 20:46'!
content

	^ content ifNil: [ | len |
		content := (len := self contentLength) 
			"unknown content length"
			ifNil: [ cstream upToEnd ] 
			ifNotNil: [ cstream next: (len asNumber) ].
		cstream close.
		content ]! !

!SCouchHTTPResponse methodsFor: 'accessing' stamp: 'Igor.Stasenko 12/27/2009 06:48'!
contentLength

	^ header at: 'content-length' ifAbsent: [nil]! !

!SCouchHTTPResponse methodsFor: 'initialize-release' stamp: 'Igor.Stasenko 3/16/2010 04:24'!
contentStream: aStream
	"use an appropriate content reader"
	
	cstream := self isChunked 
		ifTrue: [ SCouchChunkedContentStream on: aStream ]
		ifFalse: [ aStream ]
	! !

!SCouchHTTPResponse methodsFor: 'accessing' stamp: 'Igor.Stasenko 12/27/2009 06:56'!
contentType

	^ header at: 'content-type' ifAbsent: [nil]! !

!SCouchHTTPResponse methodsFor: 'accessing' stamp: 'Igor.Stasenko 12/27/2009 06:20'!
header
	"Answer the value of header"

	^ header! !

!SCouchHTTPResponse methodsFor: 'testing' stamp: 'Igor.Stasenko 1/13/2010 20:48'!
isChunked
	^ self transferEncoding = 'chunked'! !

!SCouchHTTPResponse methodsFor: 'initialize-release' stamp: 'JesusMari 7/28/2012 11:46'!
parseHeaderLines: lines
	"divide into basic lines"
	| foldedLines i |

	"fold lines that start with spaces into the previous line"
	foldedLines := OrderedCollection new.
	lines do: [ :line |
		
		line first isSeparator ifTrue: [
			foldedLines at: foldedLines size  put: (foldedLines last, line) ]
		ifFalse: [ foldedLines add: line ] ].

	"make a dictionary mapping headers to header contents"
	header := Dictionary new.
	foldedLines do: [ :line |
		i := line indexOf: $:.
		i > 0 ifTrue: [
			header 
			at: (line copyFrom: 1 to: i-1) asLowercase 
			put: (line copyFrom: i+1 to: line size) trimSpaces ] ].
! !

!SCouchHTTPResponse methodsFor: 'accessing' stamp: 'Igor.Stasenko 1/13/2010 20:42'!
readStream
	^ cstream! !

!SCouchHTTPResponse methodsFor: 'accessing' stamp: 'Igor.Stasenko 1/13/2010 20:48'!
transferEncoding
	^ header at: 'transfer-encoding' ifAbsent: [nil ]! !

Object subclass: #SCouchUTF8Stream
	instanceVariableNames: 'stream peekChar'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SCouchDB-Core'!

!SCouchUTF8Stream commentStamp: 'Igor.Stasenko 4/7/2010 01:52' prior: 0!
This is a converter stream, which reads from raw stream
and returns a utf8-decoded characters.

(Currently, used mainly for reading JSon, hence supports only 
#next
#peek
and 
#peelFor:
)!

!SCouchUTF8Stream class methodsFor: 'as yet unclassified' stamp: 'Igor.Stasenko 3/16/2010 09:31'!
on: aStream
	^ self new stream: aStream! !

!SCouchUTF8Stream methodsFor: 'as yet unclassified' stamp: 'Igor.Stasenko 4/7/2010 01:15'!
next

	^ peekChar ifNil: [ self nextUTF8Char ]
		ifNotNil: [:c | peekChar := nil. c ]  
! !

!SCouchUTF8Stream methodsFor: 'as yet unclassified' stamp: 'JesusMari 5/3/2012 15:11'!
nextByte
	"Answer a byte, not character.
	It would be more efficient to switch stream into binary mode,
	but requires more effort"
	| ch |
	ch:=stream next.
	ch ifNil:[^nil].
	^ ch asciiValue
! !

!SCouchUTF8Stream methodsFor: 'as yet unclassified' stamp: 'JesusMari 5/3/2012 15:11'!
nextUTF8Char

	| value1 value2 value3 value4 |
	(value1 := self nextByte) ifNil: [^ nil].
	value1 <= 127 ifTrue: [
		"1-byte character"
		^ Character value: value1
	].

	"at least 2-byte character"
	(value2 := self nextByte) ifNil: [^self errorMalformedInput].
	value1 <= 2r11011111 ifTrue: [
		^ Unicode charFromUnicode: ((value1 bitAnd: 31) bitShift: 6) + (value2 bitAnd: 63).
	].

	"at least 3-byte character"
	(value3 := self nextByte) ifNil: [^self errorMalformedInput].
	(value1 <= 2r11101111) ifTrue: [
		^ Unicode charFromUnicode: ((value1 bitAnd: 15) bitShift: 12) + ((value2 bitAnd: 63) bitShift: 6)
				+ (value3 bitAnd: 63).
	].

	"4-byte character"
	(value1 <= 2r11110111) ifTrue: [
		(value4 := self nextByte) ifNil: [^self errorMalformedInput].
		^ Unicode charFromUnicode: ((value1 bitAnd: 16r7) bitShift: 18) +
					((value2 bitAnd: 63) bitShift: 12) + 
					((value3 bitAnd: 63) bitShift: 6) +
					(value4 bitAnd: 63).
	].

	^self errorMalformedInput
! !

!SCouchUTF8Stream methodsFor: 'as yet unclassified' stamp: 'Igor.Stasenko 4/7/2010 01:16'!
peek
	^ peekChar ifNil: [ peekChar := self nextUTF8Char ]! !

!SCouchUTF8Stream methodsFor: 'as yet unclassified' stamp: 'Igor.Stasenko 4/7/2010 01:32'!
peekFor: anObject
	"Answer false and do not move over the next element if it is not equal to 
	the argument, anObject, or if the receiver is at the end. Answer true 
	and increment the position for accessing elements, if the next element is 
	equal to anObject."

	^ (self peek = anObject) and: [ self next. true ]! !

!SCouchUTF8Stream methodsFor: 'as yet unclassified' stamp: 'JesusMar 4/11/2012 17:48'!
stream: aStream
	stream := aStream! !

Error subclass: #SCouchDBError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SCouchDB-Core'!

!String class methodsFor: '*SCouchDB-Core' stamp: 'Igor.Stasenko 5/13/2010 05:17'!
instantiateFromCouchDocument: json
	"Translate a json object, a couch document, into an instance of receiver.
	See #asCouchDocument. "
	
	^ json at: 'sq_value'! !

!Collection class methodsFor: '*SCouchDB-Core' stamp: 'Igor.Stasenko 1/12/2010 00:43'!
instantiateFromCouchDocument: json
	"Translate a json object, a couch document, into an instance of receiver.
	See #asCouchDocument. "
	
	^ self withAll: (json at: 'sq_values') ! !

!Collection methodsFor: '*SCouchDB-Core' stamp: 'Igor.Stasenko 1/12/2010 00:21'!
asCouchDocument

	^ JsonObject new
		at: 'sq_class' put: self class name;
		at: 'sq_values' put: self;
		yourself! !
SCouchHTTPCodes initialize!
